/// <reference types="ws" />
import { h, Dict, Universal, Bot, Context, Awaitable, Adapter, Schema, WebSocketLayer, Quester } from 'koishi';
import { UpPacketsMap } from '@hieuzest/adapter-forward';
export const kForward: unique symbol;
export const kUniversalMethods: unique symbol;
export const kInternalMethods: unique symbol;
export class TimeoutError extends Error {
    constructor(args: Dict, url: string);
}
export function parseElementObject(content: any): any;
export function parseElementObjects(content: any[]): any[];
export function prepareElement(bot: Bot, element: h): Promise<h>;
export function prepareUniversalMethods<K extends keyof Universal.Methods>(bot: Bot, action: K, args: Parameters<Universal.Methods[K]>): Promise<Parameters<Universal.Methods[K]>>;
export interface Internal {
    _send: <T extends keyof UpPacketsMap>(type: T, payload: UpPacketsMap[T]['payload'], rest?: Partial<UpPacketsMap[T]>, socket?: WebSocket) => Awaitable<void>;
    _call: <T extends keyof UpPacketsMap>(type: T, payload: UpPacketsMap[T]['payload'], rest?: Partial<UpPacketsMap[T]>, socket?: WebSocket) => Awaitable<any>;
    _update: (bot: Bot, socket?: WebSocket) => Promise<void>;
}
export interface ForwardBot {
    [kUniversalMethods]?: (keyof Universal.Methods)[];
    [kInternalMethods]?: string[];
}
export class ForwardBot<T extends ForwardBot.Config = ForwardBot.Config> extends Bot<T> {
    internal: Internal;
    [kForward]: boolean;
    constructor(ctx: Context, config: T);
    initialize(): Promise<void>;
    _updateUniversalMethods(): void;
}
export namespace ForwardBot {
    interface Config extends Bot.Config {
        callback: (bot: ForwardBot) => Awaitable<void>;
        universalMethods?: (keyof Universal.Methods)[];
        internalMethods?: string[];
    }
}
export interface SharedConfig<T = 'ws' | 'ws-reverse'> {
    protocol: T;
    responseTimeout?: number;
}
export class WsClient extends Adapter.WsClient<ForwardHost> {
    protected accept: typeof accept;
    prepare(bot: ForwardHost<ForwardHost.BaseConfig & WsClient.Config>): Promise<WebSocket>;
}
export namespace WsClient {
    interface Config extends SharedConfig<'ws'>, Quester.Config, Adapter.WsClient.Config {
    }
    const Config: Schema<Config>;
}
export class WsServer extends Adapter.Server<ForwardHost<ForwardHost.BaseConfig & WsServer.Config>> {
    wsServer?: WebSocketLayer;
    constructor(ctx: Context, bot: ForwardHost);
    stop(bot: ForwardHost): Promise<void>;
}
export namespace WsServer {
    interface Config extends SharedConfig<'ws-reverse'> {
        path?: string;
    }
    const Config: Schema<Config>;
}
export function accept(client: ForwardHost, socket?: WebSocket): void;
export class ForwardHost<T extends ForwardHost.Config = ForwardHost.Config> extends Bot<T> {
    constructor(ctx: Context, config: T);
    validateSid(sid: string): boolean;
    getBots(): ForwardBot[];
    getBot(sid: string): ForwardBot;
    addBot(sid: string, rest: Partial<ForwardBot.Config>): Promise<ForwardBot>;
    removeBot(bot: ForwardBot): boolean;
}
export namespace ForwardHost {
    interface BaseConfig extends Bot.Config {
        sids: string[];
        token?: string;
    }
    const BaseConfig: Schema<BaseConfig>;
    type Config = BaseConfig & (WsServer.Config | WsClient.Config);
    const Config: Schema<Config>;
}
export default ForwardHost;
